//+-------------------------------------------------------------------------------------+
//|                                                                CurrencyTriangle.mq4 |
//|                                                                           Scriptong |
//|                                                               http://autograf.dp.ua |
//+-------------------------------------------------------------------------------------+
#property copyright "Scriptong"
#property link      "http://autograf.dp.ua"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Red
#property indicator_color2 Blue
#property indicator_color3 Lime
#property indicator_level1 0.0

//--- input parameters
extern string    RaterValute = "JPY";
extern color     LinesColor = Blue;
//--- buffers
double Master[];
double Slave[];
double Rater[];

#define Valutes 8 

bool Activate;
string MasterValute,                               //  ,  .
                                                   // ..    
       SlaveValute,                                //  ,  ..
                                                   // ..   
       Pair[3];                                    //  :
                                                   //  0 - MasterValute+SlaveValute
                                                   //  1 - MasterValute  RaterValute ..
                                                   //      .. 
                                                   //  2 - SlaveValute  RaterValute ..
                                                   //      .. 
string Valute[Valutes] = {"USD", "GBP", "EUR",     //  ,   
                          "CAD", "JPY", "AUD", "NZD", "CHF"};// .. 

int CorrSign[3, 3];                                
//      ().   (0)  ..
// ..  Pair[0],  - Pair[1],  - Pair[2].  ..
// ..   (, ).  ,    ..
// ..     .    ..
// ..  MasterValute,  - SlaveValute,  - RaterValute.
//     ,       0,  ..
// ..  ,  1,   ,  -1.
// ,     EURUSD (Pair[0]),  MasterValute = EUR, ..
// ..SlaveValute = USD.  CorrSign    :
//   CorrSign[0, 0] = 1 ( - EURUSD,  EUR   )
//   CorrSign[0, 1] = -1 ( - EURUSD,  USD   )
//   CorrSign[0, 2] = 0 ( - EURUSD,  RaterValute   )
//   CorrSign[1, 0] = 1 ( - EURRaterValute,  EUR   )
//   CorrSign[1, 1] = 0 ( - EURRaterValute,  USD )
//   CorrSign[1, 2] = -1 ( - EURRaterValute,  RaterValute  )
//   CorrSign[2, 0] = 0 ( - USDRaterValute,  EUR )
//   CorrSign[2, 1] = 1 ( - USDRaterValute,  USD   )
//   CorrSign[2, 2] = -1 ( - USDRaterValute,  RaterValute  )

double Changes[3],                                 //   , ..
                                                   // ..   
                                                   //  0 -  MasterValute
                                                   //  1 -  SlaveValute
                                                   //  2 -  RaterValute
       _Point[3],                                  //       
       _Digits[3];                                 //  

string prefix = "CURTRI_",
       line_1_name = "Line1",
       line_2_name = "Line2";  
datetime Line1Time,                                //    
         Line2Time,                                //    
         MinDepth;                                 //    ,..
                                                   // ..   ..
                                                   // .. 

//+-------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                            |
//+-------------------------------------------------------------------------------------+
int init()
{
   Activate = False;
// - 1 - ==         ======
   MasterValute = StringSubstr(Symbol(), 0, 3);    //    
   SlaveValute = StringSubstr(Symbol(), 3, 3);     //    
   CorrSign[0, 0] = 1;                             //  MasterValute  ..
                                                   // ..  Pair[0]
   CorrSign[0, 1] = -1;                            //  SlaveValute  ..
                                                   // ..  Pair[0]
   CorrSign[0, 2] = 0;                             //  RaterValute  ..
                                                   // ..   Pair[0]
   if (!IsInList(MasterValute))                    //  ""  
      return(0);                                                   
   if (!IsInList(SlaveValute))                     //   ""
      return(0); 
   if (!IsInList(RaterValute))                     //    
      return(0);                                   // ..
// - 1 - ==    ============================================================

// - 2 - ==       ==========================
   Pair[0] = Symbol();                             //   - 
   Pair[1] = GeneratePair(MasterValute, RaterValute);//    
   if (Pair[1] == "")                              //    ,  ..
      return(0);                                   // .. 
   Pair[2] = GeneratePair(SlaveValute, RaterValute);//    
   if (Pair[2] == "")                              //    ,  ..
      return(0);                                   // .. 
// - 2 - ==    ============================================================

// - 3 - ==        ================
   MinDepth = Time[Bars-1];                        //    
   datetime temp = GetDepth(Pair[1]);              //     
   if (temp == 0)                                  //   0,   
      return(0);
   MinDepth = MathMax(MinDepth, temp);             //   
   temp = GetDepth(Pair[2]);                       //     
   if (temp == 0)                                  //   0,   
      return(0);
   MinDepth = MathMax(MinDepth, temp);             //   
// - 3 - ==    ============================================================

// - 4 - ==       ==================
   Line1Time = 1;
   Line2Time = 1;
   ShowVLine(line_1_name, MathMax(MinDepth, Time[100]));
   ShowVLine(line_2_name, Time[1]);
// - 4 - ==    ============================================================

// - 5 - ==         ========================
   for (int i = 0; i < 3; i++)
   {
      _Point[i] = MarketInfo(Pair[i], MODE_POINT);
      _Digits[i] = MarketInfo(Pair[i], MODE_DIGITS);
   }
// - 5 - ==    ============================================================

// - 6 - ==    ==========================================
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0, Master);
   SetIndexLabel(0, MasterValute);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1, Slave);
   SetIndexLabel(1, SlaveValute);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2, Rater);
   SetIndexLabel(2, RaterValute);
// - 6 - ==    ============================================================

   Activate = True;
   
   return(0);
}
//+-------------------------------------------------------------------------------------+
//| Custom indicator deinitialization function                                          |
//+-------------------------------------------------------------------------------------+
int deinit()
{
   for (int i = ObjectsTotal()-1; i >= 0; i--)     //    
      if (StringSubstr(ObjectName(i), 0, StringLen(prefix)) == prefix)
         ObjectDelete(ObjectName(i));
   return(0);
}
//+-------------------------------------------------------------------------------------+
//|                                  |
//+-------------------------------------------------------------------------------------+
bool IsInList(string symbol)
{
   for (int i = 0; i < Valutes; i++)               //    
      if (symbol == Valute[i])                     // ..  .. 
         break;                                    // .. 
   if (i >= Valutes)                               //    , ..
   {                                               // .. 
      Alert("USD, GBP, EUR, CAD, JPY, AUD, NZD, CHF");
      Alert("    , " + 
            "    :");
      return(false);
   }  
   return(true);
}
//+-------------------------------------------------------------------------------------+
//|                                             |
//+-------------------------------------------------------------------------------------+
string GeneratePair(string first, string second)
{
   string result = first + second;                 //     ..
                                                   // ..  first ..
                                                   // ..second
   if (!IsPair(result))                            //    , .. 
   {                                               // .. 
      result = second + first;
      if (!IsPair(result))                         //      , 
      {                                            // ..  
         Alert(" ,   ", first, "  ", second, 
               "  . ,       . ");
         result = "";                              //   
      }
   }
   if (first == MasterValute && second == RaterValute)//     ..
   {                                               // .. CorrSign -
      CorrSign[1, 1] = 0;                          //  SlaveValute 
      if (result == first+second)                  //    MasterValute..
      {                                            // .. ,  RaterValute - 
         CorrSign[1, 0] = 1;                       // .. 
         CorrSign[1, 2] = -1;
      } 
      else                                         //    MasterValute..
      {                                            // .. ,  RaterValute - 
         CorrSign[1, 0] = -1;                      // ..
         CorrSign[1, 2] = 1;
      }  
   }
   else                                            //    - SlaveValute..
   {                                               // .. RaterValute
      CorrSign[2, 0] = 0;                          //  MasterValute 
      if (result == first+second)                  //    SlaveValute..
      {                                            // .. ,  RaterValute - 
         CorrSign[2, 1] = 1;                       // .. 
         CorrSign[2, 2] = -1;
      } 
      else                                         //    SlaveValute..
      {                                            // .. ,  RaterValute - 
         CorrSign[2, 1] = -1;                      // ..
         CorrSign[2, 2] = 1;
      }  
   }
   return(result);                                 //  
}
//+-------------------------------------------------------------------------------------+
//|                                                    |
//+-------------------------------------------------------------------------------------+
bool IsPair(string symbol)
{
   MarketInfo(symbol, MODE_BID);                   //    ..
                                                   // ..
   if (GetLastError() == 0)                        //   ,   
      return(true);
   return(false);                                  //   , ,..
                                                   // ..   
}
//+-------------------------------------------------------------------------------------+
//|  ,                                     |
//+-------------------------------------------------------------------------------------+
datetime GetDepth(string symbol)
{
   int bars = iBars(symbol, 0);                    //  -  
   if (GetLastError() != 0)                        //    ,  
      return(0);                                   // ..
   return(iTime(symbol, 0, bars-1));                //   ,  ..
                                                   // ..    
                                                   // ..   
}
//+-------------------------------------------------------------------------------------+
//|     row_beg  row_end    point             |
//+-------------------------------------------------------------------------------------+
double RowSumm(double row_beg, double row_end, double row_dig, double point)
{
   if (MathAbs(row_beg - row_end) <= point)        //      ,..
      return(0);                                   // ..    
   int mul = -1;                                   //    
                                                   // ..
   if (row_beg > row_end)                          //     (..
   {                                               // ..  ),  ..
      double temp = row_end;                       // ..
      row_end = row_beg - point;                   //  open 
      row_beg = temp;
      mul = 1;                                     //    
   }
   if (row_beg < row_end)                          //    , ..
      row_beg += point;                            // .. open 
   int row_length = MathRound((row_end - row_beg)/point);//    
   double row_average = (row_end + row_beg)/2;     //   
   double res1 = (row_beg + row_end)*              //   
                 MathRound(row_length/2.0);
                    
   
   if (MathMod(row_length, 2) == 0)                //    -  ,..
      res1 = res1 + row_average;                   // ..     ..
                                                   // .. .  
                                                   // ..   ..
                                                   // ..
   return(res1*mul);                               //    -  ,..
                                                   // ..    
}
//+-------------------------------------------------------------------------------------+
//|                                          |
//+-------------------------------------------------------------------------------------+
void ShowVLine(string suffix_name, datetime time1)
{
   string name = prefix + suffix_name;             //   :..
                                                   // .. + 
   if (ObjectFind(name) < 0)                       //    
   {
      ObjectCreate(name, OBJ_VLINE, 0, time1, 1);  //  
      ObjectSet(name, OBJPROP_COLOR, LinesColor);  //   ..
      ObjectSet(name, OBJPROP_BACK, false);        // ..    
      ObjectSet(name, OBJPROP_STYLE, STYLE_DASH);  //  
   }
   else
      ObjectMove(name, 0, time1, 1);
}
//+-------------------------------------------------------------------------------------+
//|   ,                          |
//+-------------------------------------------------------------------------------------+
int GetVLineBarIndex(string suffix_name, datetime &old_time)

{
   string name = prefix + suffix_name;             //   :..
                                                   // .. + 
   if (ObjectFind(name) < 0)                       //    , ..
      ShowVLine(suffix_name, old_time);            // ..    
   datetime time = ObjectGet(name, OBJPROP_TIME1); //    ,..
                                                   // ..   
   if (time != old_time)                           //    ,
   {
      old_time = time;                             // ..    ..
      return(true);                                // .. true
   }
   else                                            //     false
      return(false);   
}
//+-------------------------------------------------------------------------------------+
//|                                            |
//+-------------------------------------------------------------------------------------+
int GetBarNum(int num, datetime time)
{
   int cnt = 0;                                    //    
   while (cnt < 120)                               //  -  - 120
   {
      double res = iBarShift(Pair[num], 0, time);  //   
      if (GetLastError() != 4066)                  //      
         break;                                    // ..  "..
                                                   // ..   ..
                                                   // ..",    
      Comment("    ", TimeToStr(time), 
              "  ", Pair[num], "...");
      cnt++;                                       //    
      Sleep(1000);                                 // ..    1 
   }
   Comment("");
   if (cnt < 120)                                  //    , ..
      return(res);                                 // ..  res
   else                                            //   
      return(-1);   
}
//+-------------------------------------------------------------------------------------+
//|                                     |
//+-------------------------------------------------------------------------------------+
double CalcChanges(int i, datetime b_time, datetime e_time)
{
   int b_index = GetBarNum(i, b_time);             //   
   if (b_index < 0)                                //     ..
   {                                               // ..  ,   ..
      Alert("       ", // ..
            Pair[i]);
      Activate = false;                            //  
      return(0);
   }   
   int e_index = iBarShift(Pair[i], 0, e_time);    //   
   double close = iClose(Pair[i], 0, e_index);     //   
   double open = iOpen(Pair[i], 0, b_index);       //   
   for (int k = 0; k < 3; k++)                     //   
   {
      if (CorrSign[i, k] == 1)                     //    ..
                                                   // ..  , ..
         Changes[k] += (close - open)/_Point[i];   // ..    
                                                   // ..,   ..
                                                   // ...   - , ..
                                                   // .. - 
      if (CorrSign[i, k] == -1)                    //    ..
                                                   // ..  , ..
         Changes[k] += RowSumm(open, close,        // ..    ..
                               _Digits[i], _Point[i]);// ..   ..
                                                   // ..
   }
   return(close);                                  //    
}
//+-------------------------------------------------------------------------------------+
//| Custom indicator iteration function                                                 |
//+-------------------------------------------------------------------------------------+
int start()
{  
// - 1 - ==   ? ===============================================
   if (!Activate) return(0);
// - 1 - ==   =============================================================
   
// - 2 - ==        ====================
   if (GetVLineBarIndex(line_1_name, Line1Time) || //     ..
       GetVLineBarIndex(line_2_name, Line2Time))   // ..  - 
// - 2 - ==   =============================================================
   {
// - 3 - ==        ==========================
      if (Line1Time < MinDepth || Line2Time < MinDepth)//      ..
      {                                            // ..,    ..
                                                   // ..  ,   
         Alert("       : ", 
               TimeToStr(MinDepth, TIME_DATE));
         Line1Time = MathMax(Line1Time, MinDepth); // ..    
         Line2Time = MathMax(Line2Time, MinDepth); // .. 
         ShowVLine(line_1_name, Line1Time);        //  
         ShowVLine(line_2_name, Line2Time);
      }
// - 3 - ==   =============================================================

// - 4 - ==      =========================================
      if (Line1Time > Line2Time)                   //   , ..
      {                                            // ..   ..
         datetime begin_time = Line2Time;          // ..
         datetime end_time = Line1Time;
      }
      else                                         
      {
         begin_time = Line1Time;
         end_time = Line2Time;
      }
// - 4 - ==   =============================================================

// - 5 - ==     =============================================
      ArrayInitialize(Master, EMPTY_VALUE);
      ArrayInitialize(Slave, EMPTY_VALUE);
      ArrayInitialize(Rater, EMPTY_VALUE);
// - 5 - ==   =============================================================

// - 6 - ==        ======================
      double koef[3];                              //  
      int k = iBarShift(NULL, 0, begin_time);      //   
      int end = iBarShift(NULL, 0, end_time);      //    
      for (; k >= end; k--)                        //     
      {
         ArrayInitialize(Changes, 0);              //    
         for (int i = 0; i < 3; i++)               //   
         {
            koef[i] = CalcChanges(i, begin_time, Time[k]);//  
            if (koef[i] == 0) return(0);           //   ,  
         }   
         if (CorrSign[1, 0] == -1)                 //    
            koef[1] = 1/koef[1];                   // ..   , 
                                                   // ..  ..
                                                   // .. 
         Master[k] = Changes[0]*koef[1];           //     
         if (CorrSign[2, 1] == -1)                 //      
            koef[2] = 1/koef[2];
         Slave[k] = Changes[1]*koef[2];            //     
         Rater[k] = Changes[2];                    //     
      }
// - 6 - ==   =============================================================
   }

   return(0);
}
//---------------------------------------------------------------------------------------